@@ -19,6 +19,7 @@
 module Data.FileEmbed
     ( -- * Embed at compile time
       embedFile
+    , embedFileIfExists
     , embedOneFileOf
     , embedDir
     , embedDirListing
@@ -38,6 +39,7 @@ module Data.FileEmbed
     , injectFileWith
       -- * Relative path manipulation
     , makeRelativeToProject
+    , makeRelativeToLocationPredicate
       -- * Internal
     , stringToBs
     , bsToExp
@@ -60,17 +62,20 @@ import qualified Data.ByteString.Internal as B
 #endif
 import System.Directory (doesDirectoryExist, doesFileExist,
                          getDirectoryContents, canonicalizePath)
-import Control.Exception (throw, ErrorCall(..))
-import Control.Monad (filterM)
+import Control.Exception (throw, tryJust, ErrorCall(..))
+import Control.Monad (filterM, guard)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as B8
 import Control.Arrow ((&&&), second)
 import Control.Applicative ((<$>))
 import Data.ByteString.Unsafe (unsafePackAddressLen)
+import System.IO.Error (isDoesNotExistError)
 import System.IO.Unsafe (unsafePerformIO)
 import System.FilePath ((</>), takeDirectory, takeExtension)
 import Data.String (fromString)
 import Prelude as P
+import Data.List (sortBy)
+import Data.Ord (comparing)
 
 -- | Embed a single file in your source code.
 --
@@ -85,6 +90,33 @@ embedFile fp =
 #endif
   (runIO $ B.readFile fp) >>= bsToExp
 
+-- | Maybe embed a single file in your source code depending on whether or not file exists.
+--
+-- Warning: When a build is compiled with the file missing, a recompile when the file exists might not trigger an embed of the file.
+-- You might try to fix this by doing a clean build.
+--
+-- > import qualified Data.ByteString
+-- >
+-- > maybeMyFile :: Maybe Data.ByteString.ByteString
+-- > maybeMyFile = $(embedFileIfExists "dirName/fileName")
+--
+-- @since 0.0.14.0
+embedFileIfExists :: FilePath -> Q Exp
+embedFileIfExists fp = do
+  mbs <- runIO maybeFile
+  case mbs of
+    Nothing -> [| Nothing |]
+    Just bs -> do
+#if MIN_VERSION_template_haskell(2,7,0)
+      qAddDependentFile fp
+#endif
+      [| Just $(bsToExp bs) |]
+  where
+    maybeFile :: IO (Maybe B.ByteString)
+    maybeFile = 
+      either (const Nothing) Just <$> 
+      tryJust (guard . isDoesNotExistError) (B.readFile fp)
+
 -- | Embed a single existing file in your source code
 -- out of list a list of paths supplied.
 --
@@ -234,7 +266,7 @@ fileList' realTop top = do
              mapM (liftPair2 . second B.readFile)
     dirs <- filterM (doesDirectoryExist . snd) all' >>=
             mapM (fileList' realTop . fst)
-    return $ concat $ files : dirs
+    return $ sortBy (comparing fst) $ concat $ files : dirs
 
 liftPair2 :: Monad m => (a, m b) -> m (a, b)
 liftPair2 (a, b) = b >>= \b' -> return (a, b')
@@ -377,7 +409,19 @@ available, you can use the non-@With@ variants.
 --
 -- @since 0.0.10
 makeRelativeToProject :: FilePath -> Q FilePath
-makeRelativeToProject rel = do
+makeRelativeToProject = makeRelativeToLocationPredicate $ (==) ".cabal" . takeExtension
+
+-- | Take a predicate to infer the project root and a relative file path, the given file path is then attached to the inferred project root
+--
+-- This function looks at the source location of the Haskell file calling it,
+-- finds the first parent directory with a file matching the given predicate, and uses that as the
+-- root directory for fixing the relative path.
+--
+-- @$(makeRelativeToLocationPredicate ((==) ".cabal" . takeExtension) "data/foo.txt" >>= embedFile)@
+--
+-- @since 0.0.15.0
+makeRelativeToLocationPredicate :: (FilePath -> Bool) -> FilePath -> Q FilePath
+makeRelativeToLocationPredicate isTargetFile rel = do
     loc <- qLocation
     runIO $ do
         srcFP <- canonicalizePath $ loc_filename loc
@@ -392,8 +436,6 @@ makeRelativeToProject rel = do
             then return Nothing
             else do
                 contents <- getDirectoryContents dir
-                if any isCabalFile contents
+                if any isTargetFile contents
                     then return (Just dir)
                     else findProjectDir dir
-
-    isCabalFile fp = takeExtension fp == ".cabal"
